﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:54:18 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("utils.lua")

-------------------------------------------------------------------------------
-- UTILITY FUNCTIONS
-------------------------------------------------------------------------------


-- Import new contents from Source into existing table Dest
function SpeakinSpell:AddTable(Dest, Source)
	-- create the destination if it doesn't exist
	if Dest == nil then
		Dest = {}
	end
	assert( type(Dest) == "table" )
	
	-- if there's no source, then there's nothing to add
	if Source == nil then
		-- nothing to add
		return Dest
	end
	
	-- add the contents of Source, one item at a time
	for k,v in pairs(Source) do
		-- dig into embedded tables to add those too, not just create references
		if type(v) == "table" then
			-- NOTE: safety checks above will ensure we create Dest[k] if necessary
			Dest[k] = self:AddTable(Dest[k], v)
		else
			Dest[k] = v
		end
	end
	
	return Dest
end


-- create a deep copy of the table and return it
function SpeakinSpell:CopyTable(SourceTable)
	-- adding SourceTable to a new empty table will create a copy
	return self:AddTable(nil, SourceTable)
end

-- add an array of strings
function SpeakinSpell:AddStringArray(Dest, Source)
	for _,s in ipairs(Source) do
		tinsert( Dest, s )
	end
end



-- make sure there are no empty strings or empty array indices in the string list
-- also remove duplicates
function SpeakinSpell:StringArray_Compress( List )
	-- the easiest thing to do is make a new list
	-- build a reverse lookup table of the target array first, to search for duplicates
	-- TODOFUTURE: could do this a faster way i'm sure
	local NewList = {}
	local UniqueCheck = {}
	for i,s in pairs( List ) do
		if (s ~= "") and (not UniqueCheck[s]) then
			local ret = tinsert( NewList, s )
			UniqueCheck[ s ] = true
		end
	end
	return NewList
end


function SpeakinSpell:IsTableEmpty( t )
	if not t then
		return true
	end
	for _,_ in pairs(t) do
		return false
	end
	return true
end



function SpeakinSpell:GetGuildName()
	local guildName = GetGuildInfo( UnitName("player") )
	if guildName then
		return guildName
	else
		return L["Your friends"]
	end
end


function SpeakinSpell:GetPlayerFullTitleName()
	local title = GetTitleName( GetCurrentTitle() )
	-- if we don't have a title, then self:StartsWith will error on the nil value
	if not title then
		return UnitName("player")
	end
	-- we have a title, so decide if it comes first or last
	-- blizzard does this by setting a space at the beginning or end for easier string concatenation
	if self:StartsWith(title, " ") then
		return UnitName("player")..title
	else
		return title..UnitName("player")
	end
end


function SpeakinSpell:GetDefaultTarget(ShowDebugMsg)
	-- try the currently selected target
	local target = UnitName("target")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <selected>:"..target) end
		return target
	end
	
	-- nobody is selected, try focus
	target = UnitName("focus")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <focus>:"..target) end
		return target
	end
	
	-- nobody on focus, try mouseover
	target = UnitName("mouseover")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <mouseover>:"..target) end
		return target
	end
	
	-- assume self-cast ability
	target = UnitName("player")
	if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <player>:"..target) end
	
	return target
end


function SpeakinSpell:TitleCase(s)
	local first = string.sub(s,1,1)
	local rest = string.sub(s,2,-1)

	first = string.upper(first)
	
	rest = string.gsub(rest, " %a",
		function(match) 
			return string.upper(match)
		end
	)
	
	local result = first..rest
	return result
end


function SpeakinSpell:StartsWith(s, prefix)
	start = string.sub(s, 1, string.len(prefix))
	return (start == prefix)
end


function SpeakinSpell:EndsWith(s, suffix)
	strend = string.sub(s, -1 * string.len(suffix))
	return (strend == suffix)
end



function SpeakinSpell:IsInWintergraspBattle()
	--only applies to active battles
	
	--we only care if a battle is in progress
	--if we're in WG for VOA PVE and the battle is over, switch to raid settings
	if (GetWintergraspWaitTime() ~= nil) then
		-- a battle is not in progress
		return false
	end
	
	--TODOFUTURE: this has a localization problem since it does a string comparison
	-- unfortunately there doesn't appear to be a locale-independant way to test for wintergrasp
	if GetZoneText() == L["Wintergrasp GetZoneText"] then
		return true
	else
		return false
	end
end


function SpeakinSpell:GetRandomTableEntry( t, last )
	if not t then return nil end
	
	local max = #(t)
	if not (max >= 1) then return nil end
	
	local n = math.random(1, max);
	local sel = t[n]
	
	-- avoid repeating the same message we used last time
	-- unless there's only one in the list
	-- or we don't have a 'last'
	if last and (max > 1) then
		while sel == last do
			n = math.random(1, max);
			sel = t[n]
		end
	end
	
	return sel
end



function SpeakinSpell:GetScenarioKey()
	local funcname = "GetScenarioKey"
	
	-- NOTE: the EventTableEntry.Channels table stores channel names that are compatible with SendChatMessage
	--	see also CurrentMessagesGUI_OnChannelSelect
	if (IsActiveBattlefieldArena()) then
		return "Arena"
	elseif (GetNumBattlefieldScores() > 0) then
		return "BG"
	elseif (self:IsInWintergraspBattle()) then --only applies to active battles
		-- NOTE: raid warnings not enabled for WG battles
		return "WG"
	elseif (GetNumRaidMembers() > 0) then
		-- override Raid with Leader if applicable
		if IsRaidLeader() then
			return "RaidLeader"
		elseif IsRaidOfficer() then
			return "RaidOfficer"
		else
			return "Raid"
		end
	elseif (GetNumPartyMembers() > 0) then
		-- override Party with Leader if applicable
		if IsPartyLeader() then
			return "PartyLeader"
		else
			return "Party"
		end
	else -- solo
		return "Solo"
	end
end



function SpeakinSpell:GetLanguageList()
	--TODO: use this function somewhere
	--		this is just some sample code captured from wowwiki while I'm looking at it
	
	--Create and return a list of RP languages that are known to the player
	Langs = {}
	for i=1, GetNumLanguages() do
		tinsert(Langs, GetLanguageByIndex(i))
	end
	return Langs
end


-- if any key is missing from obj, it will be imported from DefaultObject
function SpeakinSpell:ValidateObject( obj, DefaultObject )
	for key,val in pairs(DefaultObject) do
		if obj[key] == nil then -- NOTE: don't replace "false" values
			if type( DefaultObject[key] ) == "table" then
				obj[key] = self:CopyTable(DefaultObject[key])
			else
				obj[key] = DefaultObject[key]
			end
		--no, this would bring in whole lists of things like messages
		--elseif type( DefaultObject[key] ) == "table" then
		--	self:ValidateObject( obj[key], DefaultObject[key] )
		end
	end
end

function SpeakinSpell:GetAddonMemoryUsedString()
	UpdateAddOnMemoryUsage("SpeakinSpell")
	return string.format( L["Memory Used: %d kb"], GetAddOnMemoryUsage("SpeakinSpell") )
end
